# Generated by Django 3.2.9 on 2022-04-24 09:40

from django.db import migrations, models

import quantityfield.fields


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="BigIntFieldSaveModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.BigIntegerQuantityField(
                        base_units="gram", unit_choices=["gram"]
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ChoicesDefinedInModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "weight",
                    quantityfield.fields.QuantityField(
                        base_units="kilogram", unit_choices=["milligram", "pounds"]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ChoicesDefinedInModelInt",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "weight",
                    quantityfield.fields.IntegerQuantityField(
                        base_units="kilogram", unit_choices=["milligram", "pounds"]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="CustomUregDecimalHayBale",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "custom_decimal",
                    quantityfield.fields.DecimalQuantityField(
                        base_units="custom",
                        decimal_places=2,
                        max_digits=10,
                        unit_choices=["custom"],
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="CustomUregHayBale",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "custom",
                    quantityfield.fields.QuantityField(
                        base_units="custom", unit_choices=["custom"]
                    ),
                ),
                (
                    "custom_int",
                    quantityfield.fields.IntegerQuantityField(
                        base_units="custom", unit_choices=["custom"]
                    ),
                ),
                (
                    "custom_bigint",
                    quantityfield.fields.BigIntegerQuantityField(
                        base_units="custom", unit_choices=["custom"]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DecimalFieldSaveModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.DecimalQuantityField(
                        base_units="gram",
                        decimal_places=2,
                        max_digits=10,
                        unit_choices=["gram"],
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="EmptyHayBaleBigInt",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.BigIntegerQuantityField(
                        base_units="gram", null=True, unit_choices=["gram"]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="EmptyHayBaleDecimal",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.DecimalQuantityField(
                        base_units="gram",
                        decimal_places=2,
                        max_digits=10,
                        null=True,
                        unit_choices=["gram"],
                    ),
                ),
                (
                    "compare",
                    models.DecimalField(decimal_places=2, max_digits=10, null=True),
                ),
            ],
        ),
        migrations.CreateModel(
            name="EmptyHayBaleFloat",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.QuantityField(
                        base_units="gram", null=True, unit_choices=["gram"]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="EmptyHayBaleInt",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.IntegerQuantityField(
                        base_units="gram", null=True, unit_choices=["gram"]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="EmptyHayBalePositiveInt",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.PositiveIntegerQuantityField(
                        base_units="gram", null=True, unit_choices=["gram"]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="FloatFieldSaveModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.QuantityField(
                        base_units="gram", unit_choices=["gram"]
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="HayBale",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.QuantityField(
                        base_units="gram", unit_choices=["gram"]
                    ),
                ),
                (
                    "weight_int",
                    quantityfield.fields.IntegerQuantityField(
                        base_units="gram", blank=True, null=True, unit_choices=["gram"]
                    ),
                ),
                (
                    "weight_bigint",
                    quantityfield.fields.BigIntegerQuantityField(
                        base_units="gram", blank=True, null=True, unit_choices=["gram"]
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="IntFieldSaveModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.IntegerQuantityField(
                        base_units="gram", unit_choices=["gram"]
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



# Generated by Django 4.2.5 on 2023-09-25 08:24

from django.db import migrations, models

import quantityfield.fields


class Migration(migrations.Migration):
    dependencies = [
        ("dummyapp", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="OffsetUnitFloatFieldSaveModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.QuantityField(
                        base_units="degC", unit_choices=["degC"]
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from django.test import TestCase

from pint import DimensionalityError

import quantityfield.fields as fields
import quantityfield.helper as helper
from quantityfield.units import ureg


class TestMatchingUnitDimensionsHelper(TestCase):
    def test_valid_choices(self):
        helper.check_matching_unit_dimension(ureg, "meter", ["mile", "foot", "cm"])

    def test_invalid_choices(self):
        with self.assertRaises(DimensionalityError):
            helper.check_matching_unit_dimension(
                ureg, "meter", ["mile", "foot", "cm", "kg"]
            )


class TestEdgeCases(TestCase):
    def test_fix_unit_registry(self):
        field = fields.IntegerQuantityField("meter")
        with self.assertRaises(ValueError):
            field.fix_unit_registry(1)

    def test_get_prep_value(self):
        field = fields.IntegerQuantityField("meter")
        with self.assertRaises(ValueError):
            field.get_prep_value("foobar")



from django.forms.widgets import MultiWidget, NumberInput, Select

import pint
from numbers import Number

from .units import ureg


class QuantityWidget(MultiWidget):
    def __init__(self, *, attrs=None, base_units=None, allowed_types=None):
        self.ureg = ureg
        choices = self.get_choices(allowed_types)
        self.base_units = base_units
        attrs = attrs or {}
        attrs.setdefault("step", "any")
        widgets = (NumberInput(attrs=attrs), Select(attrs=attrs, choices=choices))
        super(QuantityWidget, self).__init__(widgets, attrs)

    def get_choices(self, allowed_types=None):
        allowed_types = allowed_types or dir(self.ureg)
        return [(x, x) for x in allowed_types]

    def decompress(self, value):
        """This function is called during rendering

        It is responsible to split values for the two widgets
        """
        if isinstance(value, Number):
            # We assume that the given value is a proper number,
            # ready to be rendered
            return [value, self.base_units]
        elif isinstance(value, pint.Quantity):
            return [value.magnitude, value.units]

        return [None, self.base_units]



from django.db import models
from django.db.models import DecimalField

import quantityfield.fields
from quantityfield.fields import (
    BigIntegerQuantityField,
    DecimalQuantityField,
    IntegerQuantityField,
    QuantityField,
)


class FieldSaveModel(models.Model):
    name = models.CharField(max_length=20)
    weight = ...

    class Meta:
        abstract = True


class FloatFieldSaveModel(FieldSaveModel):
    weight = QuantityField("gram")


class IntFieldSaveModel(FieldSaveModel):
    weight = IntegerQuantityField("gram")


class BigIntFieldSaveModel(FieldSaveModel):
    weight = BigIntegerQuantityField("gram")


class DecimalFieldSaveModel(FieldSaveModel):
    weight = DecimalQuantityField("gram", max_digits=10, decimal_places=2)


class HayBale(models.Model):
    name = models.CharField(max_length=20)
    weight = QuantityField("gram")
    weight_int = IntegerQuantityField("gram", blank=True, null=True)
    weight_bigint = BigIntegerQuantityField("gram", blank=True, null=True)


class EmptyHayBaleFloat(models.Model):
    name = models.CharField(max_length=20)
    weight = QuantityField("gram", null=True)


class EmptyHayBaleInt(models.Model):
    name = models.CharField(max_length=20)
    weight = IntegerQuantityField("gram", null=True)


class EmptyHayBalePositiveInt(models.Model):
    name = models.CharField(max_length=20)
    weight = quantityfield.fields.PositiveIntegerQuantityField("gram", null=True)


class EmptyHayBaleBigInt(models.Model):
    name = models.CharField(max_length=20)
    weight = BigIntegerQuantityField("gram", null=True)


class EmptyHayBaleDecimal(models.Model):
    name = models.CharField(max_length=20)
    weight = DecimalQuantityField("gram", null=True, max_digits=10, decimal_places=2)
    # Value to compare with default implementation
    compare = DecimalField(max_digits=10, decimal_places=2, null=True)


class CustomUregHayBale(models.Model):
    # Custom is defined in settings in conftest.py
    custom = QuantityField("custom")
    custom_int = IntegerQuantityField("custom")
    custom_bigint = BigIntegerQuantityField("custom")


class CustomUregDecimalHayBale(models.Model):
    custom_decimal = DecimalQuantityField("custom", max_digits=10, decimal_places=2)


class ChoicesDefinedInModel(models.Model):
    weight = QuantityField("kilogram", unit_choices=["milligram", "pounds"])


class ChoicesDefinedInModelInt(models.Model):
    weight = IntegerQuantityField("kilogram", unit_choices=["milligram", "pounds"])


class OffsetUnitFloatFieldSaveModel(FieldSaveModel):
    # Note: This is a temperature not a weight.
    #       We wanted to reuse existing test cases inheritance
    weight = QuantityField("degC")



import django_pint
import quantityfield


def test_version_import_quantityfield() -> None:
    """The quantityfield version is a defined version"""
    assert quantityfield.__version__ != "unknown"
    assert quantityfield.__version__[0].isnumeric()


def test_version_import_django_pint() -> None:
    """The django_pint version is a defined version"""
    assert django_pint.__version__ != "unknown"
    assert django_pint.__version__[0].isnumeric()



import pytest

from django import forms
from django.test import TestCase

from decimal import Decimal

from tests.dummyapp.forms import (
    DefaultFormBigInt,
    DefaultFormDecimal,
    DefaultFormFieldsBigInt,
    DefaultFormFieldsDecimal,
    DefaultFormFieldsFloat,
    DefaultFormFieldsInt,
    DefaultFormFloat,
    DefaultFormInt,
    DefaultWidgetsFormBigInt,
    DefaultWidgetsFormDecimal,
    DefaultWidgetsFormFloat,
    DefaultWidgetsFormInt,
)


class IntegrationTestBase:
    DEFAULT_FORM = DefaultFormFloat
    DEFAULT_FIELDS_FORM = DefaultFormFieldsFloat
    DEFAULT_WIDGET_FORM = DefaultWidgetsFormFloat

    INPUT_STR = "10.3"
    OUTPUT_MAGNITUDE = 10.3

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Make sure we did no mistake creating the tests
        assert self.DEFAULT_FORM.Meta.model == self.DEFAULT_FIELDS_FORM.Meta.model
        assert self.DEFAULT_FORM.Meta.model == self.DEFAULT_WIDGET_FORM.Meta.model

    def _check_form_and_saved_object(self, form: forms.ModelForm, has_magnitude: bool):
        self.assertTrue(form.is_valid())
        if has_magnitude:
            self.assertAlmostEqual(
                form.cleaned_data["weight"].magnitude, self.OUTPUT_MAGNITUDE
            )
            self.assertEqual(str(form.cleaned_data["weight"].units), "gram")
        else:
            self.assertAlmostEqual(form.cleaned_data["weight"], self.OUTPUT_MAGNITUDE)
        form.save()
        obj = form.Meta.model.objects.last()
        self.assertEqual(str(obj.weight.units), "gram")
        if isinstance(self.OUTPUT_MAGNITUDE, float):
            self.assertAlmostEqual(obj.weight.magnitude, self.OUTPUT_MAGNITUDE)
        else:
            self.assertEqual(obj.weight.magnitude, self.OUTPUT_MAGNITUDE)
        self.assertIsInstance(obj.weight.magnitude, type(self.OUTPUT_MAGNITUDE))

    @pytest.mark.django_db
    def test_widget_valid_inputs_with_units(self):
        form = self.DEFAULT_FORM(
            data={
                "name": "testing",
                "weight_0": self.INPUT_STR,
                "weight_1": "gram",
            }
        )
        self._check_form_and_saved_object(form, True)

    @pytest.mark.django_db
    def test_widget_single_inputs_with_units_and_default_form_fields(self):
        """
        Test with default form fields, will still create the correct
        database entries
        """
        form = self.DEFAULT_FIELDS_FORM(
            data={
                "name": "testing",
                "weight": self.INPUT_STR,
            }
        )
        self._check_form_and_saved_object(form, False)


class TestFloatFieldWidgetIntegration(IntegrationTestBase, TestCase):
    pass


class TestDecimalFieldWidgetIntegration(IntegrationTestBase, TestCase):
    DEFAULT_FORM = DefaultFormDecimal
    DEFAULT_FIELDS_FORM = DefaultFormFieldsDecimal
    DEFAULT_WIDGET_FORM = DefaultWidgetsFormDecimal
    INPUT_STR = "10"
    OUTPUT_MAGNITUDE = Decimal("10")


class IntegrationTestBaseInt(IntegrationTestBase):
    INPUT_STR = "10"
    OUTPUT_MAGNITUDE = 10


class TestIntFiledWidgetIntegration(IntegrationTestBaseInt, TestCase):
    DEFAULT_FORM = DefaultFormInt
    DEFAULT_FIELDS_FORM = DefaultFormFieldsInt
    DEFAULT_WIDGET_FORM = DefaultWidgetsFormInt


class TestBigIntFiledWidgetIntegration(IntegrationTestBaseInt, TestCase):
    DEFAULT_FORM = DefaultFormBigInt
    DEFAULT_FIELDS_FORM = DefaultFormFieldsBigInt
    DEFAULT_WIDGET_FORM = DefaultWidgetsFormBigInt



from django.contrib import admin

from .models import *  # noqa: F401, F403


class ReadOnlyEditing(admin.ModelAdmin):
    def get_readonly_fields(self, request, obj=None):
        if obj is not None:
            return list(self.get_fields(request))
        return []


admin.site.register(BigIntFieldSaveModel, ReadOnlyEditing)
admin.site.register(ChoicesDefinedInModel, ReadOnlyEditing)
admin.site.register(ChoicesDefinedInModelInt, ReadOnlyEditing)
admin.site.register(CustomUregDecimalHayBale, ReadOnlyEditing)
admin.site.register(CustomUregHayBale, ReadOnlyEditing)
admin.site.register(DecimalFieldSaveModel, ReadOnlyEditing)
admin.site.register(EmptyHayBaleBigInt, ReadOnlyEditing)
admin.site.register(EmptyHayBaleDecimal, ReadOnlyEditing)
admin.site.register(EmptyHayBaleFloat, ReadOnlyEditing)
admin.site.register(EmptyHayBaleInt, ReadOnlyEditing)
admin.site.register(FloatFieldSaveModel, ReadOnlyEditing)
admin.site.register(HayBale, ReadOnlyEditing)
admin.site.register(IntFieldSaveModel, ReadOnlyEditing)
admin.site.register(OffsetUnitFloatFieldSaveModel, ReadOnlyEditing)



import pytest

import django.contrib.admin
from django.contrib.admin import ModelAdmin
from django.db.models import Model
from django.forms import Field, ModelForm

from typing import Dict

from quantityfield.widgets import QuantityWidget
from tests.dummyapp import models


@pytest.mark.parametrize(
    "model, field",
    [
        (models.FloatFieldSaveModel, "weight"),
        (models.IntFieldSaveModel, "weight"),
        (models.BigIntFieldSaveModel, "weight"),
        (models.DecimalFieldSaveModel, "weight"),
        (models.HayBale, "weight"),
        (models.HayBale, "weight_int"),
        (models.HayBale, "weight_bigint"),
        (models.EmptyHayBaleFloat, "weight"),
        (models.EmptyHayBaleInt, "weight"),
        (models.EmptyHayBaleBigInt, "weight"),
        (models.EmptyHayBaleDecimal, "weight"),
        (models.CustomUregHayBale, "custom"),
        (models.CustomUregHayBale, "custom_int"),
        (models.CustomUregHayBale, "custom_bigint"),
        (models.CustomUregDecimalHayBale, "custom_decimal"),
        (models.ChoicesDefinedInModel, "weight"),
        (models.ChoicesDefinedInModelInt, "weight"),
    ],
)
def test_admin_widgets(model: Model, field: str):
    """
    Test that all admin pages deliver the correct widget
    """
    admin: ModelAdmin = django.contrib.admin.site._registry[model]
    form: ModelForm = admin.get_form({})()
    form_fields: Dict[str, Field] = form.fields
    assert type(form_fields[field].widget) == QuantityWidget  # noqa



import pytest

import django.core.exceptions
import django.core.validators
from django.core.serializers import deserialize, serialize
from django.db import transaction
from django.db.models import Field, Model
from django.test import TestCase

import json
import warnings
from decimal import Decimal
from pint import DimensionalityError, UndefinedUnitError, UnitRegistry
from typing import Type, Union

from quantityfield.fields import (
    BigIntegerQuantityField,
    DecimalQuantityField,
    IntegerQuantityField,
    PositiveIntegerQuantityField,
    QuantityField,
    QuantityFieldMixin,
)
from quantityfield.units import ureg
from tests.dummyapp.models import (
    BigIntFieldSaveModel,
    CustomUregDecimalHayBale,
    CustomUregHayBale,
    DecimalFieldSaveModel,
    EmptyHayBaleBigInt,
    EmptyHayBaleDecimal,
    EmptyHayBaleFloat,
    EmptyHayBaleInt,
    EmptyHayBalePositiveInt,
    FieldSaveModel,
    FloatFieldSaveModel,
    IntFieldSaveModel,
    OffsetUnitFloatFieldSaveModel,
)

Quantity = ureg.Quantity


class BaseMixinTestFieldCreate:
    # The field that needs to be tested
    FIELD: Type[Union[Field, QuantityFieldMixin]]
    # Some fields, i.e. the decimal require default kwargs to work properly
    DEFAULT_KWARGS = {}

    def test_sets_units(self):
        test_grams = self.FIELD("gram", **self.DEFAULT_KWARGS)
        self.assertEqual(test_grams.units, ureg.gram)

    def test_fails_with_unknown_units(self):
        with self.assertRaises(UndefinedUnitError):
            test_crazy_units = self.FIELD(  # noqa: F841
                "zinghie", **self.DEFAULT_KWARGS
            )

    def test_base_units_is_required(self):
        with self.assertRaises(TypeError):
            no_units = self.FIELD(**self.DEFAULT_KWARGS)  # noqa: F841

    def test_base_units_set_with_name(self):
        okay_units = self.FIELD(base_units="meter", **self.DEFAULT_KWARGS)  # noqa: F841

    def test_base_units_are_invalid(self):
        with self.assertRaises(ValueError):
            wrong_units = self.FIELD(None, **self.DEFAULT_KWARGS)  # noqa: F841

    def test_unit_choices_must_be_valid_units(self):
        with self.assertRaises(UndefinedUnitError):
            self.FIELD(base_units="mile", unit_choices=["gunzu"], **self.DEFAULT_KWARGS)

    def test_unit_choices_must_match_base_dimensionality(self):
        with self.assertRaises(DimensionalityError):
            self.FIELD(
                base_units="gram",
                unit_choices=["meter", "ounces"],
                **self.DEFAULT_KWARGS
            )


class TestFloatFieldCreate(BaseMixinTestFieldCreate, TestCase):
    FIELD = QuantityField


class TestIntegerFieldCreate(BaseMixinTestFieldCreate, TestCase):
    FIELD = IntegerQuantityField


class TestBigIntegerFieldCreate(BaseMixinTestFieldCreate, TestCase):
    FIELD = BigIntegerQuantityField


class TestPositiveIntegerFieldCreate(BaseMixinTestFieldCreate, TestCase):
    FIELD = PositiveIntegerQuantityField


class TestDecimalFieldCreate(BaseMixinTestFieldCreate, TestCase):
    FIELD = DecimalQuantityField
    DEFAULT_KWARGS = {"max_digits": 10, "decimal_places": 2}


@pytest.mark.parametrize(
    "max_digits, decimal_places, error",
    [
        (None, None, "Invalid initialization.*expect.*integers.*"),
        (10, None, "Invalid initialization.*expect.*integers.*"),
        (None, 2, "Invalid initialization.*expect.*integers.*"),
        (-1, 2, "Invalid initialization.*positive.*larger than decimal_places.*"),
        (2, -1, "Invalid initialization.*positive.*larger than decimal_places.*"),
        (2, 3, "Invalid initialization.*positive.*larger than decimal_places.*"),
    ],
)
def test_decimal_init_fail(max_digits, decimal_places, error):
    with pytest.raises(ValueError, match=error):
        DecimalQuantityField(
            "meter", max_digits=max_digits, decimal_places=decimal_places
        )


@pytest.mark.parametrize("max_digits, decimal_places", [(2, 0), (2, 2), (1, 0)])
def decimal_init_success(max_digits, decimal_places):
    DecimalQuantityField("meter", max_digits=max_digits, decimal_places=decimal_places)


@pytest.mark.django_db
class TestCustomDecimalUreg(TestCase):
    def setUp(self):
        # Custom Values are fined in confest.py
        CustomUregDecimalHayBale.objects.create(custom_decimal=Decimal("5"))
        CustomUregDecimalHayBale.objects.create(
            custom_decimal=Decimal("5") * ureg.kilocustom,
        )

    def tearDown(self):
        CustomUregHayBale.objects.all().delete()

    def test_custom_ureg_decimal(self):
        obj = CustomUregDecimalHayBale.objects.first()
        self.assertEqual(str(obj.custom_decimal), "5.00 custom")

        obj = CustomUregDecimalHayBale.objects.last()
        self.assertEqual(str(obj.custom_decimal), "5000.00 custom")


@pytest.mark.django_db
class TestCustomUreg(TestCase):
    def setUp(self):
        # Custom Values are fined in confest.py
        CustomUregHayBale.objects.create(custom=5, custom_int=5, custom_bigint=5)
        CustomUregHayBale.objects.create(
            custom=5 * ureg.kilocustom,
            custom_int=5 * ureg.kilocustom,
            custom_bigint=5 * ureg.kilocustom,
        )

    def tearDown(self):
        CustomUregHayBale.objects.all().delete()

    def test_custom_ureg_float(self):
        obj = CustomUregHayBale.objects.first()
        self.assertIsInstance(obj.custom, ureg.Quantity)
        self.assertEqual(str(obj.custom), "5.0 custom")

        obj = CustomUregHayBale.objects.last()
        self.assertEqual(str(obj.custom), "5000.0 custom")

    def test_custom_ureg_int(self):
        obj = CustomUregHayBale.objects.first()
        self.assertIsInstance(obj.custom_int, ureg.Quantity)
        self.assertEqual(str(obj.custom_int), "5 custom")

        obj = CustomUregHayBale.objects.last()
        self.assertEqual(str(obj.custom_int), "5000 custom")

    def test_custom_ureg_bigint(self):
        obj = CustomUregHayBale.objects.first()
        self.assertIsInstance(obj.custom_int, ureg.Quantity)
        self.assertEqual(str(obj.custom_bigint), "5 custom")

        obj = CustomUregHayBale.objects.last()
        self.assertEqual(str(obj.custom_bigint), "5000 custom")


class BaseMixinNullAble:
    EMPTY_MODEL: Type[Model]
    FLOAT_SET_STR = "707.7"
    FLOAT_SET = float(FLOAT_SET_STR)
    DB_FLOAT_VALUE_EXPECTED = 707.7

    def setUp(self):
        self.EMPTY_MODEL.objects.create(name="Empty")

    def tearDown(self) -> None:
        self.EMPTY_MODEL.objects.all().delete()

    def test_accepts_assigned_null(self):
        new = self.EMPTY_MODEL()
        new.weight = None
        new.name = "Test"
        new.save()
        self.assertIsNone(new.weight)
        # Also get it from database to verify
        from_db = self.EMPTY_MODEL.objects.last()
        self.assertIsNone(from_db.weight)

    def test_accepts_auto_null(self):
        empty = self.EMPTY_MODEL.objects.first()
        self.assertIsNone(empty.weight, None)

    def test_accepts_default_pint_unit(self):
        new = self.EMPTY_MODEL(name="DefaultPintUnitTest")
        units = UnitRegistry()
        new.weight = 5 * units.kilogram
        # Different Registers so we expect a warning!
        with self.assertWarns(RuntimeWarning):
            new.save()
        obj = self.EMPTY_MODEL.objects.last()
        self.assertEqual(obj.name, "DefaultPintUnitTest")
        self.assertEqual(obj.weight.units, "gram")
        self.assertEqual(obj.weight.magnitude, 5000)

    def test_accepts_default_app_unit(self):
        new = self.EMPTY_MODEL(name="DefaultAppUnitTest")
        new.weight = 5 * ureg.kilogram
        # Make sure that the correct argument does not raise a warning
        with warnings.catch_warnings(record=True) as w:
            new.save()
        assert len(w) == 0
        obj = self.EMPTY_MODEL.objects.last()
        self.assertEqual(obj.name, "DefaultAppUnitTest")
        self.assertEqual(obj.weight.units, "gram")
        self.assertEqual(obj.weight.magnitude, 5000)

    def test_accepts_assigned_whole_number(self):
        new = self.EMPTY_MODEL(name="WholeNumber")
        new.weight = 707
        new.save()
        obj = self.EMPTY_MODEL.objects.last()
        self.assertEqual(obj.name, "WholeNumber")
        self.assertEqual(obj.weight.units, "gram")
        self.assertEqual(obj.weight.magnitude, 707)

    def test_accepts_assigned_float_number(self):
        new = self.EMPTY_MODEL(name="FloatNumber")
        new.weight = self.FLOAT_SET
        new.save()
        obj = self.EMPTY_MODEL.objects.last()
        self.assertEqual(obj.name, "FloatNumber")
        self.assertEqual(obj.weight.units, "gram")
        # We expect the database to deliver the correct type, at least
        # for postgresql this is true
        self.assertEqual(obj.weight.magnitude, self.DB_FLOAT_VALUE_EXPECTED)
        self.assertIsInstance(obj.weight.magnitude, type(self.DB_FLOAT_VALUE_EXPECTED))

    def test_serialisation(self):
        serialized = serialize(
            "json",
            [
                self.EMPTY_MODEL.objects.first(),
            ],
        )
        deserialized = json.loads(serialized)
        obj = deserialized[0]["fields"]
        self.assertEqual(obj["name"], "Empty")
        self.assertIsNone(obj["weight"])
        obj_generator = deserialize("json", serialized, ignorenonexistent=True)
        obj_back = next(obj_generator)
        self.assertEqual(obj_back.object.name, "Empty")
        self.assertIsNone(obj_back.object.weight)


@pytest.mark.django_db
class TestNullableFloat(BaseMixinNullAble, TestCase):
    EMPTY_MODEL = EmptyHayBaleFloat


@pytest.mark.django_db
class TestNullableInt(BaseMixinNullAble, TestCase):
    EMPTY_MODEL = EmptyHayBaleInt
    DB_FLOAT_VALUE_EXPECTED = int(BaseMixinNullAble.FLOAT_SET)


@pytest.mark.django_db
class TestNullablePositiveInt(BaseMixinNullAble, TestCase):
    EMPTY_MODEL = EmptyHayBalePositiveInt
    DB_FLOAT_VALUE_EXPECTED = int(BaseMixinNullAble.FLOAT_SET)

    def test_raises_validation_error_for_negative_numbers(self):
        # Test copied and modified from Django IntegerField tests
        min_value = 0
        instance = self.EMPTY_MODEL(weight=min_value - 1, name="lowest")
        expected_message = django.core.validators.MinValueValidator.message % {
            "limit_value": min_value,
        }
        with self.assertRaisesMessage(
            django.core.exceptions.ValidationError, expected_message
        ):
            instance.full_clean()

        instance.weight = min_value
        instance.full_clean()


@pytest.mark.django_db
class TestNullableBigInt(BaseMixinNullAble, TestCase):
    EMPTY_MODEL = EmptyHayBaleBigInt
    DB_FLOAT_VALUE_EXPECTED = int(BaseMixinNullAble.FLOAT_SET)


@pytest.mark.django_db
class TestNullableDecimal(BaseMixinNullAble, TestCase):
    EMPTY_MODEL = EmptyHayBaleDecimal
    DB_FLOAT_VALUE_EXPECTED = Decimal(BaseMixinNullAble.FLOAT_SET_STR)

    def test_with_default_implementation(self):
        new = self.EMPTY_MODEL(name="FloatNumber")
        new.weight = self.FLOAT_SET
        new.compare = self.FLOAT_SET
        new.save()
        obj = self.EMPTY_MODEL.objects.last()
        self.assertEqual(obj.name, "FloatNumber")
        self.assertEqual(obj.weight.units, "gram")
        # We compare with the reference implementation of django, this should
        # be always true no matter which database is used
        self.assertEqual(obj.weight.magnitude, obj.compare)
        self.assertIsInstance(obj.weight.magnitude, type(obj.compare))

    def test_with_decimal(self):
        new = self.EMPTY_MODEL(name="FloatNumber")
        new.weight = Decimal(self.FLOAT_SET_STR)
        new.compare = Decimal(self.FLOAT_SET_STR)
        new.save()
        obj = self.EMPTY_MODEL.objects.last()
        self.assertEqual(obj.name, "FloatNumber")
        self.assertEqual(obj.weight.units, "gram")
        # We compare with the reference implementation of django, this should
        # be always true no matter which database is used
        self.assertEqual(obj.weight.magnitude, obj.compare)
        self.assertIsInstance(obj.weight.magnitude, type(obj.compare))
        # But we also expect (at least for postgresql) that this a Decimal
        self.assertEqual(obj.weight.magnitude, self.DB_FLOAT_VALUE_EXPECTED)
        self.assertIsInstance(obj.weight.magnitude, Decimal)


class FieldSaveTestBase:
    MODEL: Type[FieldSaveModel]
    EXPECTED_TYPE: Type = float
    DEFAULT_WEIGHT = 100
    DEFAULT_WEIGHT_STR = "100.0"
    DEFAULT_WEIGHT_QUANTITY_STR = "100.0 gram"
    HEAVIEST = 1000
    LIGHTEST = 1
    OUNCE_VALUE = 3.52739619496
    COMPARE_QUANTITY = Quantity(0.8 * ureg.ounce)  # 1 ounce = 28.34 grams

    def setUp(self):
        self.MODEL.objects.create(
            weight=self.DEFAULT_WEIGHT,
            name="grams",
        )
        self.lightest = self.MODEL.objects.create(weight=self.LIGHTEST, name="lightest")
        self.heaviest = self.MODEL.objects.create(weight=self.HEAVIEST, name="heaviest")

    def tearDown(self):
        self.MODEL.objects.all().delete()

    def test_fails_with_incompatible_units(self):
        # we have to wrap this in a transaction
        # fixing a unit test problem
        # http://stackoverflow.com/questions/21458387/transactionmanagementerror-you-cant-execute-queries-until-the-end-of-the-atom
        metres = Quantity(100 * ureg.meter)
        with transaction.atomic():
            with self.assertRaises(DimensionalityError):
                self.MODEL.objects.create(weight=metres, name="Should Fail")

    def test_value_stored_as_quantity(self):
        obj = self.MODEL.objects.first()
        self.assertIsInstance(obj.weight, Quantity)
        self.assertEqual(str(obj.weight), self.DEFAULT_WEIGHT_QUANTITY_STR)

    def test_value_stored_as_correct_magnitude_type(self):
        obj = self.MODEL.objects.first()
        self.assertIsInstance(obj.weight, Quantity)
        self.assertIsInstance(obj.weight.magnitude, self.EXPECTED_TYPE)

    def test_value_conversion(self):
        obj = self.MODEL.objects.first()
        ounces = obj.weight.to(ureg.ounce)
        self.assertAlmostEqual(ounces.magnitude, self.OUNCE_VALUE)
        self.assertEqual(ounces.units, ureg.ounce)

    def test_order_by(self):
        qs = list(self.MODEL.objects.all().order_by("weight"))
        self.assertEqual(qs[0].name, "lightest")
        self.assertEqual(qs[-1].name, "heaviest")
        self.assertEqual(qs[0], self.lightest)
        self.assertEqual(qs[-1], self.heaviest)

    def test_comparison_with_number(self):
        qs = self.MODEL.objects.filter(weight__gt=2)
        self.assertNotIn(self.lightest, qs)

    def test_comparison_with_quantity(self):
        weight = Quantity(20 * ureg.gram)
        qs = self.MODEL.objects.filter(weight__gt=weight)
        self.assertNotIn(self.lightest, qs)

    def test_comparison_with_quantity_respects_units(self):
        qs = self.MODEL.objects.filter(weight__gt=self.COMPARE_QUANTITY)
        self.assertNotIn(self.lightest, qs)

    def test_comparison_is_actually_numeric(self):
        qs = self.MODEL.objects.filter(weight__gt=1.0)
        self.assertNotIn(self.lightest, qs)

    def test_serialisation(self):
        serialized = serialize(
            "json",
            [
                self.MODEL.objects.first(),
            ],
        )
        deserialized = json.loads(serialized)
        obj = deserialized[0]["fields"]
        self.assertEqual(obj["weight"], self.DEFAULT_WEIGHT_STR)


class FloatLikeFieldSaveTestBase(FieldSaveTestBase):
    OUNCES = Quantity(10 * ureg.ounce)
    OUNCES_IN_GRAM = 283.49523125

    def test_stores_value_in_base_units(self):
        self.MODEL.objects.create(weight=self.OUNCES, name="ounce")
        item = self.MODEL.objects.get(name="ounce")
        self.assertEqual(item.weight.units, "gram")
        self.assertAlmostEqual(item.weight.magnitude, self.OUNCES_IN_GRAM)


class TestFloatFieldSave(FloatLikeFieldSaveTestBase, TestCase):
    MODEL = FloatFieldSaveModel


class TestDecimalFieldSave(FloatLikeFieldSaveTestBase, TestCase):
    MODEL = DecimalFieldSaveModel
    DEFAULT_WEIGHT_STR = "100.00"
    DEFAULT_WEIGHT_QUANTITY_STR = "100.00 gram"
    OUNCES = Decimal("10") * ureg.ounce
    OUNCE_VALUE = Decimal("3.52739619496")
    OUNCES_IN_GRAM = Decimal("283.50")
    EXPECTED_TYPE = Decimal


class IntLikeFieldSaveTestBase(FieldSaveTestBase):
    DEFAULT_WEIGHT_STR = "100"
    DEFAULT_WEIGHT_QUANTITY_STR = "100 gram"
    EXPECTED_TYPE = int
    # 1 ounce = 28.34 grams -> we use something that can be stored as int
    COMPARE_QUANTITY = Quantity(28 * 1000 * ureg.milligram)

    @pytest.mark.xfail(reason="Not anymore supported")
    def test_store_integer_loss_of_precision(self):
        # We don't support this anymore, as it introduces to many edge cases
        # Also the normal int field accepts floats, so this should be handled
        # by the forms!
        with transaction.atomic():
            with self.assertRaisesRegex(ValueError, "loss of precision"):
                self.MODEL(name="x", weight=Quantity(10 * ureg.ounce)).save()


class TestIntFieldSave(IntLikeFieldSaveTestBase, TestCase):
    MODEL = IntFieldSaveModel


class TestBigIntFieldSave(IntLikeFieldSaveTestBase, TestCase):
    MODEL = BigIntFieldSaveModel


class TestOffsetUnitFieldSaveTestBase(FloatLikeFieldSaveTestBase, TestCase):
    MODEL = OffsetUnitFloatFieldSaveModel
    DEFAULT_WEIGHT_QUANTITY_STR = "100.0 degree_Celsius"
    HEAVIEST = 1000
    LIGHTEST = 1
    FAHRENHEIT_VALUE = 212  # 100 celsius = 212 fahrenheit
    COMPARE_QUANTITY = Quantity(100, ureg.fahrenheit)

    # Note: weight here is a temperature,
    #       reused the field name to allow inheritance of float test cae

    def test_value_conversion(self):
        obj = self.MODEL.objects.first()
        degF = obj.weight.to(ureg.fahrenheit)  # weight is in celsius
        self.assertAlmostEqual(degF.magnitude, self.FAHRENHEIT_VALUE)
        self.assertEqual(degF.units, ureg.fahrenheit)

    def test_value_stored_as_quantity(self):
        obj = self.MODEL.objects.first()
        self.assertIsInstance(obj.weight, Quantity)
        self.assertEqual(str(obj.weight), self.DEFAULT_WEIGHT_QUANTITY_STR)

    def test_stores_value_in_base_units(self):
        self.MODEL.objects.create(weight=self.FAHRENHEIT_VALUE, name="fahrenheit")
        item = self.MODEL.objects.get(name="fahrenheit")
        self.assertEqual(item.weight.units, "degree_Celsius")
        self.assertAlmostEqual(item.weight.magnitude, self.FAHRENHEIT_VALUE)

    def test_comparison_with_quantity(self):
        weight = Quantity(20, ureg.celsius)
        qs = self.MODEL.objects.filter(weight__gt=weight)
        self.assertNotIn(self.lightest, qs)

    def test_comparison_with_quantity_respects_units(self):
        qs = self.MODEL.objects.filter(weight__gt=self.COMPARE_QUANTITY)
        self.assertNotIn(self.lightest, qs)



# flake8: noqa: F841
import pytest

from django import forms
from django.test import TestCase

from decimal import Decimal
from pint import DimensionalityError, UndefinedUnitError

from quantityfield.fields import IntegerQuantityFormField, QuantityFormField
from quantityfield.units import ureg
from quantityfield.widgets import QuantityWidget
from tests.dummyapp.models import (
    ChoicesDefinedInModel,
    ChoicesDefinedInModelInt,
    HayBale,
)

Quantity = ureg.Quantity


class HayBaleForm(forms.ModelForm):
    weight = QuantityFormField(base_units="gram", unit_choices=["ounce", "gram"])
    weight_int = IntegerQuantityFormField(
        base_units="gram", unit_choices=["ounce", "gram", "kilogram"]
    )

    class Meta:
        model = HayBale
        exclude = ["weight_bigint"]


class HayBaleFormDefaultWidgets(forms.ModelForm):
    weight = QuantityFormField(
        base_units="gram", unit_choices=["ounce", "gram"], widget=forms.NumberInput
    )
    weight_int = IntegerQuantityFormField(
        base_units="gram", unit_choices=["ounce", "gram"], widget=forms.NumberInput
    )

    class Meta:
        model = HayBale
        exclude = ["weight_bigint"]


class UnitChoicesDefinedInModelFieldModelForm(forms.ModelForm):
    class Meta:
        model = ChoicesDefinedInModel
        fields = ["weight"]


class UnitChoicesDefinedInModelFieldModelFormInt(forms.ModelForm):
    class Meta:
        model = ChoicesDefinedInModelInt
        fields = ["weight"]


class NullableWeightForm(forms.Form):
    weight = QuantityFormField(base_units="gram", required=False)


class UnitChoicesForm(forms.Form):
    distance = QuantityFormField(
        base_units="kilometer", unit_choices=["mile", "kilometer", "yard", "feet"]
    )


class TestWidgets(TestCase):
    def test_creates_correct_widget_for_modelform(self):
        form = HayBaleForm()
        self.assertIsInstance(form.fields["weight"], QuantityFormField)
        self.assertIsInstance(form.fields["weight"].widget, QuantityWidget)

    def test_displays_initial_data_correctly(self):
        form = HayBaleForm(
            initial={"weight": Quantity(100 * ureg.gram), "name": "test"}
        )

    def test_clean_yields_quantity(self):
        form = HayBaleForm(
            data={
                "weight_0": 100.0,
                "weight_1": "gram",
                "weight_int_0": 100,
                "weight_int_1": "gram",
                "name": "test",
            }
        )
        self.assertTrue(form.is_valid())
        self.assertIsInstance(form.cleaned_data["weight"], Quantity)

    def test_clean_yields_quantity_in_correct_units(self):
        form = HayBaleForm(
            data={
                "weight_0": 1.0,
                "weight_1": "ounce",
                "weight_int_0": 1,
                "weight_int_1": "kilogram",
                "name": "test",
            }
        )
        self.assertTrue(form.is_valid())
        self.assertEqual(str(form.cleaned_data["weight"].units), "gram")
        self.assertAlmostEqual(form.cleaned_data["weight"].magnitude, 28.349523125)
        self.assertEqual(str(form.cleaned_data["weight_int"].units), "gram")
        self.assertAlmostEqual(form.cleaned_data["weight_int"].magnitude, 1000)

    def test_precision_lost(self):
        def test_clean_yields_quantity_in_correct_units(self):
            form = HayBaleForm(
                data={
                    "weight_0": 1.0,
                    "weight_1": "ounce",
                    "weight_int_0": 1,
                    "weight_int_1": "onuce",
                    "name": "test",
                }
            )
            self.assertFalse(form.is_valid())

    def test_base_units_is_required_for_form_field(self):
        with self.assertRaises(ValueError):
            field = QuantityFormField()  # noqa: F841

    def test_quantityfield_can_be_null(self):
        form = NullableWeightForm(data={"weight_0": None, "weight_1": None})
        self.assertTrue(form.is_valid())

    def test_validate_units(self):
        form = UnitChoicesForm(data={"distance_0": 100, "distance_1": "ounce"})
        self.assertFalse(form.is_valid())

    def test_base_units_is_included_by_default(self):
        field = QuantityFormField(base_units="mile", unit_choices=["meters", "feet"])
        self.assertIn("mile", field.units)

    def test_widget_field_displays_unit_choices(self):
        form = UnitChoicesForm()
        self.assertListEqual(
            [
                ("mile", "mile"),
                ("kilometer", "kilometer"),
                ("yard", "yard"),
                ("feet", "feet"),
            ],
            form.fields["distance"].widget.widgets[1].choices,
        )

    def test_widget_field_displays_unit_choices_for_model_field_propagation(self):
        form = UnitChoicesDefinedInModelFieldModelForm()
        self.assertListEqual(
            [
                ("kilogram", "kilogram"),
                ("milligram", "milligram"),
                ("pounds", "pounds"),
            ],
            form.fields["weight"].widget.widgets[1].choices,
        )

    def test_widget_int_field_displays_unit_choices_for_model_field_propagation(self):
        form = UnitChoicesDefinedInModelFieldModelFormInt()
        self.assertListEqual(
            [
                ("kilogram", "kilogram"),
                ("milligram", "milligram"),
                ("pounds", "pounds"),
            ],
            form.fields["weight"].widget.widgets[1].choices,
        )

    def test_unit_choices_must_be_valid_units(self):
        with self.assertRaises(UndefinedUnitError):
            field = QuantityFormField(
                base_units="mile", unit_choices=["gunzu"]
            )  # noqa: F841

    def test_unit_choices_must_match_base_dimensionality(self):
        with self.assertRaises(DimensionalityError):
            field = QuantityFormField(
                base_units="gram", unit_choices=["meter", "ounces"]
            )  # noqa: F841

    def test_widget_invalid_float(self):
        form = HayBaleForm(
            data={
                "name": "testing",
                "weight_0": "a",
                "weight_1": "gram",
                "weight_int_0": "10",
                "weight_int_1": "gram",
            }
        )
        self.assertFalse(form.is_valid())
        self.assertIn("weight", form.errors)

    def test_widget_missing_required_input(self):
        form = HayBaleForm(
            data={
                "name": "testing",
                "weight_int_0": "10",
                "weight_int_1": "gram",
            }
        )
        self.assertFalse(form.is_valid())
        self.assertIn("weight", form.errors)

    def test_widget_empty_value_for_required_input(self):
        form = HayBaleForm(
            data={
                "name": "testing",
                "weight_0": "",
                "weight_1": "gram",
                "weight_int_0": "10",
                "weight_int_1": "gram",
            }
        )
        self.assertFalse(form.is_valid())
        self.assertIn("weight", form.errors)

    def test_widget_none_value_set_for_required_input(self):
        form = HayBaleForm(
            data={
                "name": "testing",
                "weight_0": None,
                "weight_1": "gram",
                "weight_int_0": "10",
                "weight_int_1": "gram",
            }
        )
        self.assertFalse(form.is_valid())
        self.assertIn("weight", form.errors)

    def test_widget_int_precision_loss(self):
        form = HayBaleFormDefaultWidgets(
            data={
                "name": "testing",
                "weight": "10",
                "weight_int": "10.3",
            }
        )
        self.assertFalse(form.is_valid())
        self.assertTrue(form.has_error("weight_int"))


class TestWidgetRenderingBase(TestCase):
    value = 20
    expected_created = "20"
    expected_db = "20.0"

    def get_html(self, value_from_db: bool) -> str:
        """Create the rendered form with the widget"""
        bale = HayBale.objects.create(name="Fritz", weight=self.value)
        if value_from_db:
            # When creating an object django just takes the given value
            # and sets it
            # Once we receive it from the database the correct Quantity
            # is created
            bale = HayBale.objects.get(pk=bale.pk)
        form = HayBaleForm(instance=bale)
        return str(form)

    def test_widget_display(self):
        # Add to Integration tests
        html = self.get_html(False)
        expected = f'<input type="number" name="weight_0" value="{self.expected_created}" step="any" required id="id_weight_0">'
        self.assertIn(expected, html)
        self.assertIn('<option value="ounce">ounce</option>', html)

    def test_widget_display_db_value(self):
        html = self.get_html(True)
        expected = f'<input type="number" name="weight_0" value="{self.expected_db}" step="any" required id="id_weight_0">'
        self.assertIn(expected, html)
        self.assertIn('<option value="ounce">ounce</option>', html)


class TestWidgetRenderingNegativeNumber(TestWidgetRenderingBase):
    value = -20
    expected_created = "-20"
    expected_db = "-20.0"


class TestWidgetRenderingSmallNumber(TestWidgetRenderingBase):
    value = 1e-10
    expected_created = "1e-10"
    expected_db = "1e-10"


class TestWidgetRenderingZeroInt(TestWidgetRenderingBase):
    value = 0
    expected_created = "0"
    expected_db = "0.0"


class TestWidgetRenderingZeroFloat(TestWidgetRenderingBase):
    value = 0.0
    expected_created = "0.0"
    expected_db = "0.0"


class TestWidgetRenderingZeroDecimal(TestWidgetRenderingBase):
    value = Decimal(0.0)
    expected_created = "0"
    expected_db = "0.0"


class TestWidgetRenderingDecimalFromFloat(TestWidgetRenderingBase):
    # 1.0 is represenatble in base 2 and base 10, so should return 1 (not 1. + 1e-16 etc)
    value = Decimal(1.0)
    expected_created = "1"
    expected_db = "1.0"



__version__ = "0.4"
from django.conf import settings

from pint import UnitRegistry, set_application_registry

# Define default unit register
DJANGO_PINT_UNIT_REGISTER = getattr(
    settings, "DJANGO_PINT_UNIT_REGISTER", UnitRegistry()
)
# Set as default application registry for i.e. for pickle
set_application_registry(DJANGO_PINT_UNIT_REGISTER)
